@@ -27,6 +27,7 @@ from datetime import datetime
 from functools import lru_cache
 from operator import attrgetter
 
+import requests
 from ibm_watson_machine_learning.wml_client_error import WMLClientError
 from packaging import version
 
@@ -115,10 +116,17 @@ class _CredentialsProvider:
     SPACE_ID = 'space_id'
     SPACE_NAME = 'space_name'
     URL = 'url'
+    REGION = 'region'
     COS_CRN = 'cos_resource_crn'
     ML_CRN = 'ml_instance_crn'
+    REGION_TO_URL = {
+        'us-south': 'https://us-south.ml.cloud.ibm.com',
+        'eu-de': 'https://eu-de.ml.cloud.ibm.com',
+        'eu-gb': 'https://eu-gb.ml.cloud.ibm.com',
+        'jp-tok': 'https://jp-tok.ml.cloud.ibm.com',
+    }
 
-    def __init__(self, wml_credentials_file=None, wml_credentials_str=None):
+    def __init__(self, wml_credentials_file=None, wml_credentials_str=None, url=None, region=None):
         self._logger = logging.getLogger(self.__class__.__name__)
 
         if wml_credentials_str is None:
@@ -127,14 +135,14 @@ class _CredentialsProvider:
             else:
                 wml_credentials_str = self._read_wml_credentials_from_env()
             self._logger.debug('Found credential string.')
-        self.credentials = self.check_credentials(wml_credentials_str)
+        self.credentials = self.check_credentials(wml_credentials_str, url=url, region=region)
 
     def usage(self):
         print(f'${self.ENVIRONMENT_VARIABLE_NAME} should contain credentials as a Python dict of the form:')
         print(f'  {{\'{self.APIKEY}\': \'<apikey>\', \'{self.URL}\': \'https://us-south.ml.cloud.ibm.com\'}}')
         print(f'Or set ${self.ENVIRONMENT_VARIABLE_NAME_FILE} to the path to a file containing the same information.')
 
-    def check_credentials(self, wml_cred_str):
+    def check_credentials(self, wml_cred_str, url, region):
         assert type(wml_cred_str) is str
         if not wml_cred_str:
             self._logger.error('WML credentials must not be an empty string.')
@@ -147,6 +155,29 @@ class _CredentialsProvider:
             assert type(wml_credentials[self.APIKEY]) is str
         else:
             assert type(wml_credentials[self.TOKEN]) is str
+        if region:
+            if url:
+                self._logger.error(f"You must not specify both '{self.URL}' and '{self.REGION}'.")
+                raise InvalidCredentials
+            wml_credentials[self.REGION] = region
+            # Setting a region must clear the URL, otherwise there will be an
+            # ambiguity (and therefore an error) just below
+            wml_credentials.pop(self.URL)
+        if self.REGION in wml_credentials:
+            region = wml_credentials[self.REGION]
+            if self.URL in wml_credentials:
+                self._logger.error(f"WML credentials must not have both '{self.URL}' and '{self.REGION}'.")
+                self.usage()
+                raise InvalidCredentials
+            try:
+                wml_credentials[self.URL] = self.REGION_TO_URL[region]
+            except KeyError:
+                self._logger.error(f"Unknown region '{region}'.")
+                raise InvalidCredentials
+            wml_credentials.pop(self.REGION)
+        if url:
+            # The url specified takes priority over the one in the credentials, if any.
+            wml_credentials[self.URL] = url
         assert self.URL in wml_credentials
         assert type(wml_credentials[self.URL]) is str
         url = wml_credentials[self.URL]
@@ -220,15 +251,23 @@ class DOWMLLib:
 
     def __init__(self, wml_credentials_file=None,
                  space_id=None,
+                 url=None,
+                 region=None,
                  tz=datetime.utcnow().astimezone().tzinfo):
-        """Read and validate the WML credentials
+        f"""Read and validate the WML credentials
 
         Args:
             wml_credentials_file: path to the file that contains the WML credentials.
             If None, they are read from the environment.
             space_id: the id of the space that should be used. If specified, this
-            replaces the one in the credentials
-            tz: timezone to use to display time, defaults to Python's default timezone"""
+            replaces the one in the credentials.
+            url: the URL for the Machine Learning service to use. If specified, this
+            replaces the one in the credentials. url and region can't be both specified.
+            region: An alternative way to specify the URL for the Machine Learning service
+            to use. If specified, the URL will be deduced from the region. This
+            replaces the one in the credentials. url and region can't be both specified.
+            Possible values for the region are {list(_CredentialsProvider.REGION_TO_URL.keys())}
+            tz: timezone to use to display time, defaults to Python's default timezone."""
 
         self.model_type = self.MODEL_TYPES[0]
         """Type of model to solve"""
@@ -245,7 +284,7 @@ class DOWMLLib:
 
         self._logger = logging.getLogger(self.__class__.__name__)
 
-        cred_provider = _CredentialsProvider(wml_credentials_file)
+        cred_provider = _CredentialsProvider(wml_credentials_file, url=url, region=region)
         wml_credentials = cred_provider.credentials
 
         # A space name in the credentials changes the default
@@ -616,9 +655,12 @@ class DOWMLLib:
             # GH-1: This happens when the job failed
             pass
 
-    def _delete_data_assets(self, job_id):
-        job_details = self.get_job_details(job_id, with_contents='names')
-        odr = job_details['entity']['decision_optimization'].get('output_data_references', [])
+    def _delete_data_assets(self, job_details):
+        job_id = job_details['metadata']['id']
+        try:
+            odr = job_details['entity']['decision_optimization']['output_data_references']
+        except KeyError:
+            odr = []
         for output in odr:
             if output.get('type') != 'data_asset':
                 continue
@@ -635,17 +677,56 @@ class DOWMLLib:
                 except WMLClientError:
                     self._logger.error('Exception raised while trying to delete the asset.', exc_info=True)
 
+    def _client_deployments_delete_job(self, job_id, hard, job_details):
+        """Deletes the platform run, so that the deployment job is deleted as well.
+
+        If only calling
+           client.deployments.delete_job(job_id, hard)
+        the 'run' of the 'platform job' on the Watson Studio side is left,
+        and it will never be deleted.
+        On the other hand, deleting the run on the WS side also deletes the
+        deployment job on the WML side. So let's do that.
+        """
+        client = self._get_or_make_client()
+        wml_url = self._wml_credentials['url']
+        # We don't want to (try to) delete the WS run if we only cancel the job
+        everything_ok_so_far = hard
+        ws_url = client.PLATFORM_URLS_MAP.get(wml_url)
+        if not ws_url:
+            everything_ok_so_far = False
+        if everything_ok_so_far:
+            try:
+                platform_job_id = job_details['entity']['platform_job']['job_id']
+                platform_run_id = job_details['entity']['platform_job']['run_id']
+                url = f'{ws_url}/v2/jobs/{platform_job_id}/runs/{platform_run_id}?space_id={self._space_id}'
+            except KeyError:
+                everything_ok_so_far = False
+        if everything_ok_so_far:
+            r = requests.delete(url, headers={'Authorization': f'Bearer {client.service_instance._get_token()}',
+                                              'Content-Type': 'application/json',
+                                              'cache-control': 'no-cache'})
+            if r.status_code != 204:
+                self._logger.error(f'Error when trying to delete the Watson Studio run. {r.text}')
+                everything_ok_so_far = False
+        if not everything_ok_so_far:
+            if hard:
+                self._logger.error('Could not delete the Watson Studio run. Deleting the WML job deployment instead...')
+            # else:
+            #   we just wanted to cancel the job, so there's nothing to warn against
+            client.deployments.delete_job(job_id, hard)
+
     def delete_job(self, job_id, hard=False):
         """Delete the given job.
 
         :param job_id: the job to be deleted
         :param hard: if False, cancel the job. If true, delete it completely
         """
-        client = self._get_or_make_client()
+        job_details = None
         if hard:
-            self._delete_data_assets(job_id)
+            job_details = self.get_job_details(job_id, with_contents='names')
+            self._delete_data_assets(job_details)
         self._logger.debug(f'Deleting job {job_id}...')
-        client.deployments.delete_job(job_id, hard)
+        self._client_deployments_delete_job(job_id, hard, job_details)
         self._logger.debug('Done.')
 
     def decode_log(self, output):
